{-# INCLUDE "HsBase.h" #-}
{-# LINE 1 "System/CPUTime.hsc" #-}
-----------------------------------------------------------------------------
{-# LINE 2 "System/CPUTime.hsc" #-}
-- |
-- Module      :  System.CPUTime
-- Copyright   :  (c) The University of Glasgow 2001
-- License     :  BSD-style (see the file libraries/base/LICENSE)
-- 
-- Maintainer  :  libraries@haskell.org
-- Stability   :  provisional
-- Portability :  portable
--
-- The standard CPUTime library.
--
-----------------------------------------------------------------------------

module System.CPUTime 
	(
         getCPUTime,       -- :: IO Integer
	 cpuTimePrecision  -- :: Integer
        ) where

import Prelude

import Data.Ratio


{-# LINE 28 "System/CPUTime.hsc" #-}


{-# LINE 32 "System/CPUTime.hsc" #-}


{-# LINE 34 "System/CPUTime.hsc" #-}
import Foreign
import Foreign.C


{-# LINE 38 "System/CPUTime.hsc" #-}

{-# LINE 39 "System/CPUTime.hsc" #-}


{-# LINE 41 "System/CPUTime.hsc" #-}
-- -----------------------------------------------------------------------------
-- |Computation 'getCPUTime' returns the number of picoseconds CPU time
-- used by the current program.  The precision of this result is
-- implementation-dependent.

getCPUTime :: IO Integer
getCPUTime = do


{-# LINE 50 "System/CPUTime.hsc" #-}
-- getrusage() is right royal pain to deal with when targetting multiple
-- versions of Solaris, since some versions supply it in libc (2.3 and 2.5),
-- while 2.4 has got it in libucb (I wouldn't be too surprised if it was back
-- again in libucb in 2.6..)
--
-- Avoid the problem by resorting to times() instead.
--

{-# LINE 75 "System/CPUTime.hsc" #-}

{-# LINE 76 "System/CPUTime.hsc" #-}
    allocaBytes (16) $ \ p_tms -> do
{-# LINE 77 "System/CPUTime.hsc" #-}
    times p_tms
    u_ticks  <- ((\hsc_ptr -> peekByteOff hsc_ptr 0)) p_tms :: IO CClock
{-# LINE 79 "System/CPUTime.hsc" #-}
    s_ticks  <- ((\hsc_ptr -> peekByteOff hsc_ptr 4)) p_tms :: IO CClock
{-# LINE 80 "System/CPUTime.hsc" #-}
    let realToInteger = round . realToFrac :: Real a => a -> Integer
    return (( (realToInteger u_ticks + realToInteger s_ticks) * 1000000000000) 
			`div` fromIntegral clockTicks)

type CTms = ()
foreign import ccall unsafe times :: Ptr CTms -> IO CClock

{-# LINE 92 "System/CPUTime.hsc" #-}

{-# LINE 93 "System/CPUTime.hsc" #-}


{-# LINE 127 "System/CPUTime.hsc" #-}

{-# LINE 128 "System/CPUTime.hsc" #-}

-- |The 'cpuTimePrecision' constant is the smallest measurable difference
-- in CPU time that the implementation can record, and is given as an
-- integral number of picoseconds.


{-# LINE 134 "System/CPUTime.hsc" #-}
cpuTimePrecision :: Integer
cpuTimePrecision = round ((1000000000000::Integer) % fromIntegral (clockTicks))

{-# LINE 137 "System/CPUTime.hsc" #-}


{-# LINE 139 "System/CPUTime.hsc" #-}
clockTicks :: Int
clockTicks =

{-# LINE 142 "System/CPUTime.hsc" #-}
    (100)
{-# LINE 143 "System/CPUTime.hsc" #-}

{-# LINE 147 "System/CPUTime.hsc" #-}

{-# LINE 148 "System/CPUTime.hsc" #-}
